@@ -19,6 +19,8 @@ class Curve(abstract.Curve):
 
     This class provides the following properties:
 
+    * :py:attr:`type` = spline
+    * :py:attr:`id`
     * :py:attr:`order`
     * :py:attr:`degree`
     * :py:attr:`knotvector`
@@ -66,48 +68,37 @@ class Curve(abstract.Curve):
 
     Please refer to the :py:class:`.abstract.Curve()` documentation for more details.
     """
+    # __slots__ = ('_insert_knot_func', '_remove_knot_func')
 
     def __init__(self, **kwargs):
         super(Curve, self).__init__(**kwargs)
+        self._evaluator = evaluators.CurveEvaluator(find_span_func=self._span_func)
         self._insert_knot_func = kwargs.get('insert_knot_func', operations.insert_knot)
         self._remove_knot_func = kwargs.get('remove_knot_func', operations.remove_knot)
-        self._evaluator = evaluators.CurveEvaluator(find_span_func=self._span_func)
 
     def save(self, file_name):
         """  Saves the curve as a pickled file.
 
+        .. deprecated:: 5.2.4
+
+            Use :func:`.exchange.export_json()` instead.
+
         :param file_name: name of the file to be saved
         :type file_name: str
-        :raises IOError: an error occurred writing the file
         """
-        # Create a dictionary from the curve data
-        expdata = {'rational': self.rational,
-                   'degree': self.degree,
-                   'knotvector': self.knotvector,
-                   'ctrlpts': self._control_points,
-                   'dimension': self.dimension}
-
-        save_pickle(expdata, file_name)
+        return None
 
     def load(self, file_name):
         """ Loads the curve from a pickled file.
 
+        .. deprecated:: 5.2.4
+
+            Use :func:`.exchange.import_json()` instead.
+
         :param file_name: name of the file to be loaded
         :type file_name: str
-        :raises IOError: an error occurred reading the file
         """
-        impdata = read_pickle(file_name)
-
-        if self.rational != impdata['rational']:
-            raise TypeError("Curve types are not compatible (rational vs. non-rational mismatch)")
-
-        # Clean control points and evaluated points
-        self.reset(ctrlpts=True, evalpts=True)
-
-        # Set the curve data
-        self.degree = impdata['degree']
-        self.set_ctrlpts(impdata['ctrlpts'])
-        self.knotvector = impdata['knotvector']
+        return None
 
     def evaluate(self, **kwargs):
         """ Evaluates the curve.
@@ -152,13 +143,8 @@ class Curve(abstract.Curve):
         # Clean up the curve points
         self.reset(evalpts=True)
 
-        # Evaluate
-        cpts = self._evaluator.evaluate(start=start, stop=stop,
-                                        degree=self.degree, knotvector=self.knotvector,
-                                        ctrlpts=self._control_points, sample_size=self.sample_size,
-                                        dimension=self._dimension, precision=self._precision)
-
-        self._eval_points = cpts
+        # Evaluate and cache
+        self._eval_points = self._evaluator.evaluate(self.data, start=start, stop=stop)
 
     def evaluate_single(self, param):
         """ Evaluates the curve at the input parameter.
@@ -177,10 +163,7 @@ class Curve(abstract.Curve):
                 raise GeomdlException("Parameters should be between 0 and 1")
 
         # Evaluate the curve point
-        pt = self._evaluator.evaluate(start=param, stop=param,
-                                      degree=self.degree, knotvector=self.knotvector,
-                                      ctrlpts=self._control_points, sample_size=self.sample_size,
-                                      dimension=self._dimension, precision=self._precision)
+        pt = self._evaluator.evaluate(self.data, start=param, stop=param)
 
         return pt[0]
 
@@ -208,6 +191,19 @@ class Curve(abstract.Curve):
     def derivatives(self, u, order=0, **kwargs):
         """ Evaluates n-th order curve derivatives at the given parameter value.
 
+        The output of this method is list of n-th order derivatives. If ``order`` is ``0``, then it will only output
+        the evaluated point. Similarly, if ``order`` is ``2``, then it will output the evaluated point, 1st derivative
+        and the 2nd derivative. For instance;
+
+        .. code-block:: python
+
+            # Assuming a curve (crv) is defined on a parametric domain [0.0, 1.0]
+            # Let's take the curve derivative at the parametric position u = 0.35
+            ders = crv.derivatives(u=0.35, order=2)
+            ders[0]  # evaluated point, equal to crv.evaluate_single(0.35)
+            ders[1]  # 1st derivative at u = 0.35
+            ders[2]  @ 2nd derivative at u = 0.35
+
         :param u: parameter value
         :type u: float
         :param order: derivative order
@@ -219,9 +215,7 @@ class Curve(abstract.Curve):
         super(Curve, self).derivatives(u=u, order=order, **kwargs)
 
         # Evaluate and return the derivative at knot u
-        return self._evaluator.derivatives(parameter=u, deriv_order=order, degree=self.degree,
-                                           knotvector=self.knotvector, ctrlpts=self._control_points,
-                                           dimension=self._dimension)
+        return self._evaluator.derivatives(self.data, parpos=u, deriv_order=order)
 
     def insert_knot(self, param, **kwargs):
         """ Inserts the knot and updates the control points array and the knot vector.
@@ -287,31 +281,10 @@ class Curve(abstract.Curve):
         if check_num and self._eval_points:
             self.evaluate()
 
-    def tangent(self, param, **kwargs):
+    def tangent(self, parpos, **kwargs):
         """ Evaluates the tangent vector of the curve at the given parametric position(s).
 
-        The ``param`` argument can be
-
-        * a float value for evaluation at a single parametric position
-        * a list of float values for evaluation at the multiple parametric positions
-
-        The return value will be in the order of the input parametric position list.
-
-        This method accepts the following keyword arguments:
-
-        * ``normalize``: normalizes the output vector. Default value is *True*.
-
-        :param param: parametric position(s) where the evaluation will be executed
-        :type param: float, list or tuple
-        :return: an array containing "point" and "vector" pairs
-        :rtype: tuple
-        """
-        return operations.tangent(self, param, **kwargs)
-
-    def normal(self, parpos, **kwargs):
-        """ Evaluates the normal vector of the curve at the given parametric position(s).
-
-        The ``param`` argument can be
+        The ``parpos`` argument can be
 
         * a float value for evaluation at a single parametric position
         * a list of float values for evaluation at the multiple parametric positions
@@ -324,7 +297,28 @@ class Curve(abstract.Curve):
 
         :param parpos: parametric position(s) where the evaluation will be executed
         :type parpos: float, list or tuple
-        :return: an array containing "point" and "vector" pairs
+        :return: tangent vector as a tuple of the origin point and the vector components
+        :rtype: tuple
+        """
+        return operations.tangent(self, parpos, **kwargs)
+
+    def normal(self, parpos, **kwargs):
+        """ Evaluates the normal to the tangent vector of the curve at the given parametric position(s).
+
+        The ``parpos`` argument can be
+
+        * a float value for evaluation at a single parametric position
+        * a list of float values for evaluation at the multiple parametric positions
+
+        The return value will be in the order of the input parametric position list.
+
+        This method accepts the following keyword arguments:
+
+        * ``normalize``: normalizes the output vector. Default value is *True*.
+
+        :param parpos: parametric position(s) where the evaluation will be executed
+        :type parpos: float, list or tuple
+        :return: normal vector as a tuple of the origin point and the vector components
         :rtype: tuple
         """
         return operations.normal(self, parpos, **kwargs)
@@ -332,7 +326,7 @@ class Curve(abstract.Curve):
     def binormal(self, parpos, **kwargs):
         """ Evaluates the binormal vector of the curve at the given parametric position(s).
 
-        The ``param`` argument can be
+        The ``parpos`` argument can be
 
         * a float value for evaluation at a single parametric position
         * a list of float values for evaluation at the multiple parametric positions
@@ -345,7 +339,7 @@ class Curve(abstract.Curve):
 
         :param parpos: parametric position(s) where the evaluation will be executed
         :type parpos: float, list or tuple
-        :return: an array containing "point" and "vector" pairs
+        :return: binormal vector as a tuple of the origin point and the vector components
         :rtype: tuple
         """
         return operations.binormal(self, parpos, **kwargs)
@@ -357,6 +351,8 @@ class Surface(abstract.Surface):
 
     This class provides the following properties:
 
+    * :py:attr:`type` = spline
+    * :py:attr:`id`
     * :py:attr:`order_u`
     * :py:attr:`order_v`
     * :py:attr:`degree_u`
@@ -423,14 +419,15 @@ class Surface(abstract.Surface):
 
     Please refer to the :py:class:`.abstract.Surface()` documentation for more details.
     """
+    # __slots__ = ('_insert_knot_func', '_remove_knot_func', '_control_points2D')
 
     def __init__(self, **kwargs):
         super(Surface, self).__init__(**kwargs)
-        self._insert_knot_func = kwargs.get('insert_knot_func', operations.insert_knot)
-        self._remove_knot_func = kwargs.get('remove_knot_func', operations.remove_knot)
-        self._control_points2D = self._init_array()  # control points, 2-D array [u][v]
         self._evaluator = evaluators.SurfaceEvaluator(find_span_func=self._span_func)
         self._tsl_component = tessellate.TriangularTessellate()
+        self._control_points2D = self._init_array()  # control points, 2-D array [u][v]
+        self._insert_knot_func = kwargs.get('insert_knot_func', operations.insert_knot)
+        self._remove_knot_func = kwargs.get('remove_knot_func', operations.remove_knot)
 
     @property
     def ctrlpts2d(self):
@@ -567,45 +564,26 @@ class Surface(abstract.Surface):
     def save(self, file_name):
         """ Saves the surface as a pickled file.
 
+        .. deprecated:: 5.2.4
+
+            Use :func:`.exchange.export_json()` instead.
+
         :param file_name: name of the file to be saved
         :type file_name: str
-        :raises IOError: an error occurred writing the file
         """
-        # Create a dictionary from the surface data
-        expdata = {'rational': self.rational,
-                   'degree_u': self.degree_u,
-                   'degree_v': self.degree_v,
-                   'knotvector_u': self.knotvector_u,
-                   'knotvector_v': self.knotvector_v,
-                   'ctrlpts_size_u': self.ctrlpts_size_u,
-                   'ctrlpts_size_v': self.ctrlpts_size_v,
-                   'ctrlpts': self._control_points,
-                   'dimension': self.dimension}
-
-        save_pickle(expdata, file_name)
+        return None
 
     def load(self, file_name):
         """ Loads the surface from a pickled file.
 
+        .. deprecated:: 5.2.4
+
+            Use :func:`.exchange.import_json()` instead.
+
         :param file_name: name of the file to be loaded
         :type file_name: str
-        :raises IOError: an error occurred reading the file
         """
-        impdata = read_pickle(file_name)
-
-        # Check if we have loaded the correct type of surface
-        if self.rational != impdata['rational']:
-            raise TypeError("Surface types are not compatible (rational vs. non-rational mismatch)")
-
-        # Clean control points and evaluated points
-        self.reset(ctrlpts=True, evalpts=True)
-
-        # Set the surface data
-        self.degree_u = impdata['degree_u']
-        self.degree_v = impdata['degree_v']
-        self.set_ctrlpts(impdata['ctrlpts'], impdata['ctrlpts_size_u'], impdata['ctrlpts_size_v'])
-        self.knotvector_u = impdata['knotvector_u']
-        self.knotvector_v = impdata['knotvector_v']
+        return None
 
     def transpose(self):
         """ Transposes the surface by swapping u and v parametric directions. """
@@ -659,14 +637,10 @@ class Surface(abstract.Surface):
         # Clean up the surface points
         self.reset(evalpts=True)
 
-        # Evaluate
-        spts = self._evaluator.evaluate(start=(start_u, start_v), stop=(stop_u, stop_v),
-                                        degree=self._degree, knotvector=self._knot_vector,
-                                        ctrlpts_size=self._control_points_size, ctrlpts=self._control_points,
-                                        sample_size=self.sample_size, dimension=self._dimension,
-                                        precision=self._precision)
-
-        self._eval_points = spts
+        # Evaluate and cache
+        self._eval_points = self._evaluator.evaluate(self.data,
+                                                     start=(start_u, start_v),
+                                                     stop=(stop_u, stop_v))
 
     def evaluate_single(self, param):
         """ Evaluates the surface at the input (u, v) parameter pair.
@@ -680,11 +654,7 @@ class Surface(abstract.Surface):
         super(Surface, self).evaluate_single(param)
 
         # Evaluate the surface point
-        pt = self._evaluator.evaluate(start=param, stop=param,
-                                      degree=self._degree, knotvector=self._knot_vector,
-                                      ctrlpts_size=self._control_points_size, ctrlpts=self._control_points,
-                                      sample_size=self.sample_size, dimension=self._dimension,
-                                      precision=self._precision)
+        pt = self._evaluator.evaluate(self.data, start=param, stop=param)
 
         return pt[0]
 
@@ -730,10 +700,7 @@ class Surface(abstract.Surface):
         super(Surface, self).derivatives(u=u, v=v, order=order, **kwargs)
 
         # Evaluate and return the derivatives
-        return self._evaluator.derivatives(parameter=(u, v), deriv_order=order,
-                                           degree=self._degree, knotvector=self._knot_vector,
-                                           ctrlpts_size=self._control_points_size, ctrlpts=self._control_points,
-                                           dimension=self._dimension)
+        return self._evaluator.derivatives(self.data, parpos=(u, v), deriv_order=order)
 
     def insert_knot(self, u=None, v=None, **kwargs):
         """ Inserts knot(s) on the u- or v-directions
@@ -858,6 +825,8 @@ class Volume(abstract.Volume):
 
     This class provides the following properties:
 
+    * :py:attr:`type` = spline
+    * :py:attr:`id`
     * :py:attr:`order_u`
     * :py:attr:`order_v`
     * :py:attr:`order_w`
@@ -896,60 +865,37 @@ class Volume(abstract.Volume):
 
     Please refer to the :py:class:`.abstract.Volume()` documentation for more details.
     """
+    # __slots__ = ('_insert_knot_func', '_remove_knot_func')
 
     def __init__(self, **kwargs):
         super(Volume, self).__init__(**kwargs)
+        self._evaluator = evaluators.VolumeEvaluator(find_span_func=self._span_func)
         self._insert_knot_func = kwargs.get('insert_knot_func', operations.insert_knot)
         self._remove_knot_func = kwargs.get('remove_knot_func', operations.remove_knot)
-        self._evaluator = evaluators.VolumeEvaluator(find_span_func=self._span_func)
 
     def save(self, file_name):
         """ Saves the volume as a pickled file.
 
+        .. deprecated:: 5.2.4
+
+            Use :func:`.exchange.export_json()` instead.
+
         :param file_name: name of the file to be saved
         :type file_name: str
-        :raises IOError: an error occurred writing the file
         """
-        # Create a dictionary from the surface data
-        expdata = {'rational': self.rational,
-                   'degree_u': self.degree_u,
-                   'degree_v': self.degree_v,
-                   'degree_w': self.degree_w,
-                   'knotvector_u': self.knotvector_u,
-                   'knotvector_v': self.knotvector_v,
-                   'knotvector_w': self.knotvector_w,
-                   'ctrlpts_size_u': self.ctrlpts_size_u,
-                   'ctrlpts_size_v': self.ctrlpts_size_v,
-                   'ctrlpts_size_w': self.ctrlpts_size_w,
-                   'ctrlpts': self._control_points,
-                   'dimension': self.dimension}
-
-        save_pickle(expdata, file_name)
+        return None
 
     def load(self, file_name):
         """ Loads the volume from a pickled file.
 
+        .. deprecated:: 5.2.4
+
+            Use :func:`.exchange.import_json()` instead.
+
         :param file_name: name of the file to be loaded
         :type file_name: str
-        :raises IOError: an error occurred reading the file
         """
-        idata = read_pickle(file_name)
-
-        # Check if we have loaded the correct type of surface
-        if self.rational != idata['rational']:
-            raise TypeError("Volume types are not compatible (rational vs. non-rational mismatch)")
-
-        # Clean control points and evaluated points
-        self.reset(ctrlpts=True, evalpts=True)
-
-        # Set the surface data
-        self.degree_u = idata['degree_u']
-        self.degree_v = idata['degree_v']
-        self.degree_w = idata['degree_w']
-        self.set_ctrlpts(idata['ctrlpts'], idata['ctrlpts_size_u'], idata['ctrlpts_size_v'], idata['ctrlpts_size_w'])
-        self.knotvector_u = idata['knotvector_u']
-        self.knotvector_v = idata['knotvector_v']
-        self.knotvector_w = idata['knotvector_w']
+        return None
 
     def evaluate(self, **kwargs):
         """ Evaluates the volume.
@@ -984,13 +930,10 @@ class Volume(abstract.Volume):
         # Clean up the evaluated points
         self.reset(evalpts=True)
 
-        # Evaluate
-        vpts = self._evaluator.evaluate(start=(start_u, start_v, start_w), stop=(stop_u, stop_v, stop_w),
-                                        degree=self._degree, knotvector=self._knot_vector,
-                                        ctrlpts_size=self._control_points_size, ctrlpts=self._control_points,
-                                        sample_size=self.sample_size, dimension=self._dimension,
-                                        precision=self._precision)
-        self._eval_points = vpts
+        # Evaluate and cache
+        self._eval_points = self._evaluator.evaluate(self.data,
+                                                     start=(start_u, start_v, start_w),
+                                                     stop=(stop_u, stop_v, stop_w))
 
     def evaluate_single(self, param):
         """ Evaluates the volume at the input (u, v, w) parameter.
@@ -1009,12 +952,7 @@ class Volume(abstract.Volume):
                 raise GeomdlException("Parameters should be between 0 and 1")
 
         # Evaluate the volume point
-        pt = self._evaluator.evaluate(start=param, stop=param,
-                                      degree=self._degree, knotvector=self._knot_vector,
-                                      ctrlpts_size=self._control_points_size, ctrlpts=self._control_points,
-                                      sample_size=self.sample_size, dimension=self._dimension,
-                                      precision=self._precision)
-
+        pt = self._evaluator.evaluate(self.data, start=param, stop=param)
         return pt[0]
 
     def evaluate_list(self, param_list):
@@ -1117,50 +1055,3 @@ class Volume(abstract.Volume):
         # Evaluate curve again if it has already been evaluated before knot removal
         if check_num and self._eval_points:
             self.evaluate()
-
-
-def save_pickle(data_dict, file_name):
-    """ Saves the contents of the data dictionary as a pickled file.
-
-    Helper function for curve and surface ``save`` method.
-
-    :param data_dict: data dictionary
-    :type data_dict: dict
-    :param file_name: name of the file to be saved
-    :type file_name: str
-    :raises IOError: an error occurred writing the file
-    """
-    # Try opening the file for writing
-    try:
-        with open(file_name, 'wb') as fp:
-            # Pickle the data dictionary
-            pickle.dump(data_dict, fp)
-    except IOError as e:
-        print("An error occurred. {}".format(e.args[-1]))
-        raise e
-    except Exception:
-        raise
-
-
-def read_pickle(file_name):
-    """ Reads a data dictionary from a pickled file.
-
-    Helper function for curve and surface ``load`` method.
-
-    :param file_name: name of the file to be loaded
-    :type file_name: str
-    :return: data dictionary
-    :rtype: dict
-    :raises IOError: an error occurred reading the file
-    """
-    # Try opening the file for reading
-    try:
-        with open(file_name, 'rb') as fp:
-            # Read and return the pickled file
-            impdata = pickle.load(fp)
-            return impdata
-    except IOError as e:
-        print("An error occurred. {}".format(e.args[-1]))
-        raise e
-    except Exception:
-        raise
