@@ -5,12 +5,16 @@
 """
 
 # Library version
-__version__ = "5.2.1"
+__version__ = "5.2.10"
 
-# Author and licence
+# Author and license
 __author__ = "Onur Rauf Bingol"
 __license__ = "MIT"
 
+# Optional variables
+__description__ = 'Object-oriented B-Spline and NURBS evaluation library'
+__keywords__ = 'NURBS B-Spline curve surface CAD modeling visualization surface-generator'
+
 # Support for "from geomdl import *"
 # @see: https://stackoverflow.com/a/41895257
 # @see: https://stackoverflow.com/a/35710527
@@ -35,3 +39,8 @@ __all__ = [
     'utilities',
     'voxelize'
 ]
+
+
+def geomdl_version():
+    """ Returns geomdl full version as a tuple """
+    return tuple(__version__.split('.'))
