@@ -23,7 +23,7 @@ def find_span_binsearch(degree, knot_vector, num_ctrlpts, knot, **kwargs):
     Implementation of Algorithm A2.1 from The NURBS Book by Piegl & Tiller.
 
     The NURBS Book states that the knot span index always starts from zero, i.e. for a knot vector [0, 0, 1, 1];
-    if FindSpan returns 1, then the knot is between the interval [0, 1).
+    if FindSpan returns 1, then the knot is between the half-open interval [0, 1).
 
     :param degree: degree, :math:`p`
     :type degree: int
@@ -141,6 +141,8 @@ def basis_function(degree, knot_vector, span, knot):
     """ Computes the non-vanishing basis functions for a single parameter.
 
     Implementation of Algorithm A2.2 from The NURBS Book by Piegl & Tiller.
+    Uses recurrence to compute the basis functions, also known as Cox - de
+    Boor recursion formula.
 
     :param degree: degree, :math:`p`
     :type degree: int
@@ -170,9 +172,68 @@ def basis_function(degree, knot_vector, span, knot):
     return N
 
 
+def basis_function_one(degree, knot_vector, span, knot):
+    """ Computes the value of a basis function for a single parameter.
+
+    Implementation of Algorithm 2.4 from The NURBS Book by Piegl & Tiller.
+
+    :param degree: degree, :math:`p`
+    :type degree: int
+    :param knot_vector: knot vector
+    :type knot_vector: list, tuple
+    :param span: knot span, :math:`i`
+    :type span: int
+    :param knot: knot or parameter, :math:`u`
+    :type knot: float
+    :return: basis function, :math:`N_{i,p}`
+    :rtype: float
+    """
+    # Special case at boundaries
+    if (span == 0 and knot == knot_vector[0]) or \
+            (span == len(knot_vector) - degree - 2) and knot == knot_vector[len(knot_vector) - 1]:
+        return 1.0
+
+    # Knot is outside of span range
+    if knot < knot_vector[span] or knot >= knot_vector[span + degree + 1]:
+        return 0.0
+
+    N = [0.0 for _ in range(degree + span + 1)]
+
+    # Initialize the zeroth degree basis functions
+    for j in range(0, degree + 1):
+        if knot_vector[span + j] <= knot < knot_vector[span + j + 1]:
+            N[j] = 1.0
+
+    # Computing triangular table of basis functions
+    for k in range(1, degree + 1):
+        # Detecting zeros saves computations
+        saved = 0.0
+        if N[0] != 0.0:
+            saved = ((knot - knot_vector[span]) * N[0]) / (knot_vector[span + k] - knot_vector[span])
+
+        for j in range(0, degree - k + 1):
+            Uleft = knot_vector[span + j + 1]
+            Uright = knot_vector[span + j + k + 1]
+
+            # Zero detection
+            if N[j + 1] == 0.0:
+                N[j] = saved
+                saved = 0.0
+            else:
+                temp = N[j + 1] / (Uright - Uleft)
+                N[j] = saved + (Uright - knot) * temp
+                saved = (knot - Uleft) * temp
+
+    return N[0]
+
+
 def basis_functions(degree, knot_vector, spans, knots):
     """ Computes the non-vanishing basis functions for a list of parameters.
 
+    Wrapper for :func:`.helpers.basis_function` to process multiple span
+    and knot values. Uses recurrence to compute the basis functions, also
+    known as Cox - de Boor recursion formula.
+
     :param degree: degree, :math:`p`
     :type degree: int
     :param knot_vector: knot vector, :math:`U`
@@ -194,6 +255,13 @@ def basis_function_all(degree, knot_vector, span, knot):
     """ Computes all non-zero basis functions of all degrees from 0 up to the input degree for a single parameter.
 
     A slightly modified version of Algorithm A2.2 from The NURBS Book by Piegl & Tiller.
+    Wrapper for :func:`.helpers.basis_function` to compute multiple basis functions.
+    Uses recurrence to compute the basis functions, also known as Cox - de Boor
+    recursion formula.
+
+    For instance; if ``degree = 2``, then this function will compute the basis function
+    values of degrees **0, 1** and **2** for the ``knot`` value at the input knot ``span``
+    of the ``knot_vector``.
 
     :param degree: degree, :math:`p`
     :type degree: int
@@ -304,83 +372,6 @@ def basis_function_ders(degree, knot_vector, span, knot, order):
     return ders
 
 
-def basis_functions_ders(degree, knot_vector, spans, knots, order):
-    """ Computes derivatives of the basis functions for a list of parameters.
-
-    :param degree: degree, :math:`p`
-    :type degree: int
-    :param knot_vector: knot vector, :math:`U`
-    :type knot_vector: list, tuple
-    :param spans: list of knot spans
-    :type spans:  list, tuple
-    :param knots: list of knots or parameters
-    :type knots: list, tuple
-    :param order: order of the derivative
-    :type order: int
-    :return: derivatives of the basis functions
-    :rtype: list
-    """
-    basis_ders = []
-    for span, knot in zip(spans, knots):
-        basis_ders.append(basis_function_ders(degree, knot_vector, span, knot, order))
-    return basis_ders
-
-
-def basis_function_one(degree, knot_vector, span, knot):
-    """ Computes the value of a basis function for a single parameter.
-
-    Implementation of Algorithm 2.4 from The NURBS Book by Piegl & Tiller.
-
-    :param degree: degree, :math:`p`
-    :type degree: int
-    :param knot_vector: knot vector
-    :type knot_vector: list, tuple
-    :param span: knot span, :math:`i`
-    :type span: int
-    :param knot: knot or parameter, :math:`u`
-    :type knot: float
-    :return: basis function, :math:`N_{i,p}`
-    :rtype: float
-    """
-    # Special case at boundaries
-    if (span == 0 and knot == knot_vector[0]) or \
-            (span == len(knot_vector) - degree - 2) and knot == knot_vector[len(knot_vector) - 1]:
-        return 1.0
-
-    # Knot is outside of span range
-    if knot < knot_vector[span] or knot >= knot_vector[span + degree + 1]:
-        return 0.0
-
-    N = [0.0 for _ in range(degree + span + 1)]
-
-    # Initialize the zeroth degree basis functions
-    for j in range(0, degree + 1):
-        if knot_vector[span + j] <= knot < knot_vector[span + j + 1]:
-            N[j] = 1.0
-
-    # Computing triangular table of basis functions
-    for k in range(1, degree + 1):
-        # Detecting zeros saves computations
-        saved = 0.0
-        if N[0] != 0.0:
-            saved = ((knot - knot_vector[span]) * N[0]) / (knot_vector[span + k] - knot_vector[span])
-
-        for j in range(0, degree - k + 1):
-            Uleft = knot_vector[span + j + 1]
-            Uright = knot_vector[span + j + k + 1]
-
-            # Zero detection
-            if N[j + 1] == 0.0:
-                N[j] = saved
-                saved = 0.0
-            else:
-                temp = N[j + 1] / (Uright - Uleft)
-                N[j] = saved + (Uright - knot) * temp
-                saved = (knot - Uleft) * temp
-
-    return N[0]
-
-
 def basis_function_ders_one(degree, knot_vector, span, knot, order):
     """ Computes the derivative of one basis functions for a single parameter.
 
@@ -477,6 +468,31 @@ def basis_function_ders_one(degree, knot_vector, span, knot, order):
     return ders
 
 
+def basis_functions_ders(degree, knot_vector, spans, knots, order):
+    """ Computes derivatives of the basis functions for a list of parameters.
+
+    Wrapper for :func:`.helpers.basis_function_ders` to process multiple span
+    and knot values.
+
+    :param degree: degree, :math:`p`
+    :type degree: int
+    :param knot_vector: knot vector, :math:`U`
+    :type knot_vector: list, tuple
+    :param spans: list of knot spans
+    :type spans:  list, tuple
+    :param knots: list of knots or parameters
+    :type knots: list, tuple
+    :param order: order of the derivative
+    :type order: int
+    :return: derivatives of the basis functions
+    :rtype: list
+    """
+    basis_ders = []
+    for span, knot in zip(spans, knots):
+        basis_ders.append(basis_function_ders(degree, knot_vector, span, knot, order))
+    return basis_ders
+
+
 def knot_insertion(degree, knotvector, ctrlpts, u, **kwargs):
     """ Computes the control points of the rational/non-rational spline after knot insertion.
 
@@ -622,6 +638,10 @@ def knot_removal(degree, knotvector, ctrlpts, u, **kwargs):
     s = kwargs.get('s', find_multiplicity(u, knotvector))  # multiplicity
     r = kwargs.get('span', find_span_linear(degree, knotvector, len(ctrlpts), u))  # knot span
 
+    # Edge case
+    if num < 1:
+        return ctrlpts
+
     # Initialize variables
     first = r - degree
     last = r - s
@@ -777,6 +797,10 @@ def knot_removal_kv(knotvector, span, r):
     :return: updated knot vector
     :rtype: list
     """
+    # Edge case
+    if r < 1:
+        return knotvector
+
     # Create a deep copy of the input knot  vector
     kv_updated = deepcopy(knotvector)
 
@@ -805,12 +829,11 @@ def knot_refinement(degree, knotvector, ctrlpts, **kwargs):
     than 1, then the algorithm finds the middle knots in each internal knot span to increase the number of knots to be
     refined.
 
-    **Example**: Let the knot vector to be refined is ``[0, 2, 4]`` with the superfluous knots from the start and end
-    are removed:
+    **Example**: Let the degree is 2 and the knot vector to be refined is ``[0, 2, 4]`` with the superfluous knots
+    from the start and end are removed. Knot vectors with the changing ``density (d)`` value will be:
 
-    * If ``density`` is 1, knot vector to be refined is ``[0, 2, 4]``
-    * If ``density`` is 2, knot vector to be refined is ``[0, 1, 2, 3, 4]``
-    * If ``density`` is 3, knot vector to be refined is ``[0, 0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4]``
+    * ``d = 1``, knot vector ``[0, 1, 1, 2, 2, 3, 3, 4]``
+    * ``d = 2``, knot vector ``[0, 0.5, 0.5, 1, 1, 1.5, 1.5, 2, 2, 2.5, 2.5, 3, 3, 3.5, 3.5, 4]``
 
     Keyword Arguments:
         * ``knot_list``: knot list to be refined. *Default: list of internal knots*
@@ -848,7 +871,7 @@ def knot_refinement(degree, knotvector, ctrlpts, **kwargs):
     knot_list = sorted(set(knot_list))
 
     # Increase knot density
-    for d in range(0, density - 1):
+    for d in range(0, density):
         rknots = []
         for i in range(len(knot_list) - 1):
             knot_tmp = knot_list[i] + ((knot_list[i + 1] - knot_list[i]) / 2.0)
@@ -1008,11 +1031,16 @@ def degree_reduction(degree, ctrlpts, **kwargs):
     pts_red[-1] = ctrlpts[-1]
 
     # Find if the degree is an even or an odd number
-    p_is_odd = False if degree % 2 == 0 else True
+    p_is_odd = True if degree % 2 != 0 else False
 
     # Compute control points of degree-reduced 1-dimensional shape
     r = int((degree - 1) / 2)
-    r1 = r - 1 if p_is_odd else r
+    # Handle a special case when degree = 2
+    if degree == 2:
+        r1 = r - 2
+    else:
+        # Determine r1 w.r.t. degree evenness
+        r1 = r - 1 if p_is_odd else r
     for i in range(1, r1 + 1):
         alpha = float(i) / float(degree)
         pts_red[i] = [(c1 - (alpha * c2)) / (1 - alpha) for c1, c2 in zip(ctrlpts[i], pts_red[i - 1])]
@@ -1029,3 +1057,112 @@ def degree_reduction(degree, ctrlpts, **kwargs):
 
     # Return computed control points after degree reduction
     return pts_red
+
+
+def curve_deriv_cpts(dim, degree, kv, cpts, rs, deriv_order=0):
+    """ Compute control points of curve derivatives.
+
+    Implementation of Algorithm A3.3 from The NURBS Book by Piegl & Tiller.
+
+    :param dim: spatial dimension of the curve
+    :type dim: int
+    :param degree: degree of the curve
+    :type degree: int
+    :param kv: knot vector
+    :type kv: list, tuple
+    :param cpts: control points
+    :type cpts: list, tuple
+    :param rs: minimum (r1) and maximum (r2) knot spans that the curve derivative will be computed
+    :param deriv_order: derivative order, i.e. the i-th derivative
+    :type deriv_order: int
+    :return: control points of the derivative curve over the input knot span range
+    :rtype: list
+    """
+    r = rs[1] - rs[0]
+
+    # Initialize return value (control points)
+    PK = [[[None for _ in range(dim)] for _ in range(r + 1)] for _ in range(deriv_order + 1)]
+
+    # Algorithm A3.3
+    for i in range(0, r + 1):
+        PK[0][i][:] = [elem for elem in cpts[rs[0] + i]]
+
+    for k in range(1, deriv_order + 1):
+        tmp = degree - k + 1
+        for i in range(0, r - k + 1):
+            PK[k][i][:] = [tmp * (elem1 - elem2) /
+                           (kv[rs[0] + i + degree + 1] - kv[rs[0] + i + k]) for elem1, elem2
+                           in zip(PK[k - 1][i + 1], PK[k - 1][i])]
+
+    # Return control points (as a 2-dimensional list of points)
+    return PK
+
+
+def surface_deriv_cpts(dim, degree, kv, cpts, cpsize, rs, ss, deriv_order=0):
+    """ Compute control points of surface derivatives.
+
+    Implementation of Algorithm A3.7 from The NURBS Book by Piegl & Tiller.
+
+    :param dim: spatial dimension of the surface
+    :type dim: int
+    :param degree: degrees
+    :type degree: list, tuple
+    :param kv: knot vectors
+    :type kv: list, tuple
+    :param cpts: control points
+    :type cpts: list, tuple
+    :param cpsize: number of control points in all parametric directions
+    :type cpsize: list, tuple
+    :param rs: minimum (r1) and maximum (r2) knot spans for the u-direction
+    :type rs: list, tuple
+    :param ss: minimum (s1) and maximum (s2) knot spans for the v-direction
+    :type ss: list, tuple
+    :param deriv_order: derivative order, i.e. the i-th derivative
+    :type deriv_order: int
+    :return: control points of the derivative surface over the input knot span ranges
+    :rtype: list
+    """
+    # Initialize return value (control points)
+    PKL = [[[[[None for _ in range(dim)]
+              for _ in range(cpsize[1])] for _ in range(cpsize[0])]
+            for _ in range(deriv_order + 1)] for _ in range(deriv_order + 1)]
+
+    du = min(degree[0], deriv_order)
+    dv = min(degree[1], deriv_order)
+
+    r = rs[1] - rs[0]
+    s = ss[1] - ss[0]
+
+    # Control points of the U derivatives of every U-curve
+    for j in range(ss[0], ss[1] + 1):
+        PKu = curve_deriv_cpts(dim=dim,
+                               degree=degree[0],
+                               kv=kv[0],
+                               cpts=[cpts[j + (cpsize[1] * i)] for i in range(cpsize[0])],
+                               rs=rs,
+                               deriv_order=du)
+
+        # Copy into output as the U partial derivatives
+        for k in range(0, du + 1):
+            for i in range(0, r - k + 1):
+                PKL[k][0][i][j - ss[0]] = PKu[k][i]
+
+    # Control points of the V derivatives of every U-differentiated V-curve
+    for k in range(0, du):
+        for i in range(0, r - k + 1):
+            dd = min(deriv_order - k, dv)
+
+            PKuv = curve_deriv_cpts(dim=dim,
+                                    degree=degree[1],
+                                    kv=kv[1][ss[0]:],
+                                    cpts=PKL[k][0][i],
+                                    rs=(0, s),
+                                    deriv_order=dd)
+
+            # Copy into output
+            for l in range(1, dd + 1):
+                for j in range(0, s - l + 1):
+                    PKL[k][l][i][j] = PKuv[l][j]
+
+    # Return control points
+    return PKL
