@@ -9,7 +9,6 @@
 
 from . import vis
 import numpy as np
-import plotly
 from plotly import graph_objs
 
 
@@ -60,24 +59,7 @@ class VisConfig(vis.VisConfigAbstract):
         super(VisConfig, self).__init__(**kwargs)
         self.dtype = np.float
         # Set Plotly custom variables
-        self.figure_image_filename = "temp-figure"
-        self.figure_image_format = "png"
-        self.figure_filename = "temp-plot.html"
-
-        # Enable online plotting (default is offline plotting as it works perfectly without any issues)
-        # @see: https://plot.ly/python/getting-started/#initialization-for-online-plotting
-        online_plotting = kwargs.get('online', False)
-
-        # Detect jupyter and/or ipython environment
-        try:
-            get_ipython
-            from plotly.offline import download_plotlyjs, init_notebook_mode
-            init_notebook_mode(connected=True)
-            self.plotfn = plotly.plotly.iplot if online_plotting else plotly.offline.iplot
-            self.no_ipython = False
-        except NameError:
-            self.plotfn = plotly.plotly.plot if online_plotting else plotly.offline.plot
-            self.no_ipython = True
+        self.figure_image_filename = "temp-plot.html"
 
         # Get keyword arguments
         self.display_ctrlpts = kwargs.get('ctrlpts', True)
@@ -195,22 +177,11 @@ class VisCurve2D(vis.VisAbstract):
         fig_filename = kwargs.get('fig_save_as', None)
         fig_display = kwargs.get('display_plot', True)
 
-        # Prepare plot configuration
-        plotfn_dict = {
-            'show_link': False,
-            'filename': self.vconf.figure_filename,
-            'image': None if fig_display else self.vconf.figure_image_format,
-        }
-        if self.vconf.no_ipython:
-            plotfn_dict_extra = {
-                'image_filename': self.vconf.figure_image_filename if fig_filename is None else fig_filename,
-                'auto_open': fig_display,
-            }
-            # Python < 3.5 does not support starred expressions inside dicts
-            plotfn_dict.update(plotfn_dict_extra)
-
         # Display the plot
-        self.vconf.plotfn(fig, **plotfn_dict)
+        if fig_display:
+            fig.write_html(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
+        else:
+            fig.write_image(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
 
 
 class VisCurve3D(vis.VisAbstract):
@@ -229,6 +200,10 @@ class VisCurve3D(vis.VisAbstract):
         for plot in self._plots:
             pts = np.array(plot['ptsarr'], dtype=self.vconf.dtype)
 
+            # Try not to fail if the input is 2D
+            if pts.shape[1] == 2:
+                pts = np.c_[pts, np.zeros(pts.shape[0])]
+
             # Plot control points
             if plot['type'] == 'ctrlpts' and self.vconf.display_ctrlpts:
                 figure = graph_objs.Scatter3d(
@@ -339,25 +314,11 @@ class VisCurve3D(vis.VisAbstract):
         fig_filename = kwargs.get('fig_save_as', None)
         fig_display = kwargs.get('display_plot', True)
 
-        # Prepare plot configuration
-        plotfn_dict = {
-            'show_link': False,
-            'filename': self.vconf.figure_filename,
-            'image': None if fig_display else self.vconf.figure_image_format,
-        }
-        if self.vconf.no_ipython:
-            plotfn_dict_extra = {
-                'image_filename': self.vconf.figure_image_filename if fig_filename is None else fig_filename,
-                'auto_open': fig_display,
-            }
-            # Python < 3.5 does not support starred expressions inside dicts
-            plotfn_dict.update(plotfn_dict_extra)
-
         # Display the plot
-        self.vconf.plotfn(fig, **plotfn_dict)
-
-        # Return the figure object
-        return fig
+        if fig_display:
+            fig.write_html(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
+        else:
+            fig.write_image(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
 
 
 class VisSurface(vis.VisAbstract):
@@ -511,25 +472,11 @@ class VisSurface(vis.VisAbstract):
         fig_filename = kwargs.get('fig_save_as', None)
         fig_display = kwargs.get('display_plot', True)
 
-        # Prepare plot configuration
-        plotfn_dict = {
-            'show_link': False,
-            'filename': self.vconf.figure_filename,
-            'image': None if fig_display else self.vconf.figure_image_format,
-        }
-        if self.vconf.no_ipython:
-            plotfn_dict_extra = {
-                'image_filename': self.vconf.figure_image_filename if fig_filename is None else fig_filename,
-                'auto_open': fig_display,
-            }
-            # Python < 3.5 does not support starred expressions inside dicts
-            plotfn_dict.update(plotfn_dict_extra)
-
         # Display the plot
-        self.vconf.plotfn(fig, **plotfn_dict)
-
-        # Return the figure object
-        return fig
+        if fig_display:
+            fig.write_html(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
+        else:
+            fig.write_image(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
 
 
 class VisVolume(vis.VisAbstract):
@@ -654,22 +601,8 @@ class VisVolume(vis.VisAbstract):
         fig_filename = kwargs.get('fig_save_as', None)
         fig_display = kwargs.get('display_plot', True)
 
-        # Prepare plot configuration
-        plotfn_dict = {
-            'show_link': False,
-            'filename': self.vconf.figure_filename,
-            'image': None if fig_display else self.vconf.figure_image_format,
-        }
-        if self.vconf.no_ipython:
-            plotfn_dict_extra = {
-                'image_filename': self.vconf.figure_image_filename if fig_filename is None else fig_filename,
-                'auto_open': fig_display,
-            }
-            # Python < 3.5 does not support starred expressions inside dicts
-            plotfn_dict.update(plotfn_dict_extra)
-
         # Display the plot
-        self.vconf.plotfn(fig, **plotfn_dict)
-
-        # Return the figure object
-        return fig
+        if fig_display:
+            fig.write_html(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
+        else:
+            fig.write_image(file=self.vconf.figure_image_filename if fig_filename is None else fig_filename)
